@@ -20,7 +20,6 @@ import abc
 import argparse
 import functools
 import logging
-import re
 
 from cliff import command
 from cliff import lister
@@ -32,104 +31,26 @@ from neutronclient._i18n import _
 from neutronclient.common import exceptions
 from neutronclient.common import utils
 
-HEX_ELEM = '[0-9A-Fa-f]'
-UUID_PATTERN = '-'.join([HEX_ELEM + '{8}', HEX_ELEM + '{4}',
-                         HEX_ELEM + '{4}', HEX_ELEM + '{4}',
-                         HEX_ELEM + '{12}'])
 HYPHEN_OPTS = ['tags_any', 'not_tags', 'not_tags_any']
 
 
-def _get_resource_plural(resource, client):
-    plurals = getattr(client, 'EXTED_PLURALS', [])
-    for k in plurals:
-        if plurals[k] == resource:
-            return k
-    return resource + 's'
-
-
 def find_resource_by_id(client, resource, resource_id, cmd_resource=None,
                         parent_id=None, fields=None):
-    if not cmd_resource:
-        cmd_resource = resource
-    cmd_resource_plural = _get_resource_plural(cmd_resource, client)
-    resource_plural = _get_resource_plural(resource, client)
-    obj_lister = getattr(client, "list_%s" % cmd_resource_plural)
-    # perform search by id only if we are passing a valid UUID
-    match = re.match(UUID_PATTERN, resource_id)
-    collection = resource_plural
-    if match:
-        params = {'id': resource_id}
-        if fields:
-            params['fields'] = fields
-        if parent_id:
-            data = obj_lister(parent_id, **params)
-        else:
-            data = obj_lister(**params)
-        if data and data[collection]:
-            return data[collection][0]
-    not_found_message = (_("Unable to find %(resource)s with id "
-                           "'%(id)s'") %
-                         {'resource': resource, 'id': resource_id})
-    # 404 is raised by exceptions.NotFound to simulate serverside behavior
-    raise exceptions.NotFound(message=not_found_message)
+    return client.find_resource_by_id(resource, resource_id, cmd_resource,
+                                      parent_id, fields)
 
 
 def find_resourceid_by_id(client, resource, resource_id, cmd_resource=None,
                           parent_id=None):
-    info = find_resource_by_id(client, resource, resource_id, cmd_resource,
-                               parent_id, fields='id')
-    return info['id']
-
-
-def _find_resource_by_name(client, resource, name, project_id=None,
-                           cmd_resource=None, parent_id=None, fields=None):
-    if not cmd_resource:
-        cmd_resource = resource
-    cmd_resource_plural = _get_resource_plural(cmd_resource, client)
-    resource_plural = _get_resource_plural(resource, client)
-    obj_lister = getattr(client, "list_%s" % cmd_resource_plural)
-    params = {'name': name}
-    if fields:
-        params['fields'] = fields
-    if project_id:
-        params['tenant_id'] = project_id
-    if parent_id:
-        data = obj_lister(parent_id, **params)
-    else:
-        data = obj_lister(**params)
-    collection = resource_plural
-    info = data[collection]
-    if len(info) > 1:
-        raise exceptions.NeutronClientNoUniqueMatch(resource=resource,
-                                                    name=name)
-    elif len(info) == 0:
-        not_found_message = (_("Unable to find %(resource)s with name "
-                               "'%(name)s'") %
-                             {'resource': resource, 'name': name})
-        # 404 is raised by exceptions.NotFound to simulate serverside behavior
-        raise exceptions.NotFound(message=not_found_message)
-    else:
-        return info[0]
+    return find_resource_by_id(client, resource, resource_id, cmd_resource,
+                               parent_id, fields='id')['id']
 
 
 def find_resource_by_name_or_id(client, resource, name_or_id,
                                 project_id=None, cmd_resource=None,
                                 parent_id=None, fields=None):
-    try:
-        return find_resource_by_id(client, resource, name_or_id,
-                                   cmd_resource, parent_id, fields)
-    except exceptions.NotFound:
-        try:
-            return _find_resource_by_name(client, resource, name_or_id,
-                                          project_id, cmd_resource, parent_id,
-                                          fields)
-        except exceptions.NotFound:
-            not_found_message = (_("Unable to find %(resource)s with name "
-                                   "or id '%(name_or_id)s'") %
-                                 {'resource': resource,
-                                  'name_or_id': name_or_id})
-            raise exceptions.NotFound(
-                message=not_found_message)
+    return client.find_resource(resource, name_or_id, project_id,
+                                cmd_resource, parent_id, fields)
 
 
 def find_resourceid_by_name_or_id(client, resource, name_or_id,
@@ -211,9 +132,9 @@ def _process_previous_argument(current_arg, _value_number, current_type_str,
                                values_specs):
     if current_arg is not None:
         if _value_number == 0 and (current_type_str or _list_flag):
-                # This kind of argument should have value
-                raise exceptions.CommandError(
-                    _("Invalid values_specs %s") % ' '.join(values_specs))
+            # This kind of argument should have value
+            raise exceptions.CommandError(
+                _("Invalid values_specs %s") % ' '.join(values_specs))
         if _value_number > 1 or _list_flag or current_type_str == 'list':
             current_arg.update({'nargs': '+'})
         elif _value_number == 0:
@@ -264,6 +185,14 @@ def parse_args_to_dict(values_specs):
     current_item = None
     # the str after 'type='
     current_type_str = None
+    # dict of allowed types
+    allowed_type_dict = {
+        'bool': utils.str2bool,
+        'dict': utils.str2dict,
+        'int': int,
+        'str': str,
+    }
+
     for _item in values_specs_copy:
         if _item.startswith('--'):
             # Deal with previous argument if any
@@ -294,12 +223,16 @@ def parse_args_to_dict(values_specs):
                     _("Invalid values_specs %s") % ' '.join(values_specs))
             if 'type' not in current_arg:
                 current_type_str = _item.split('=', 2)[1]
-                current_arg.update({'type': eval(current_type_str)})
-                if current_type_str == 'bool':
-                    current_arg.update({'type': utils.str2bool})
-                elif current_type_str == 'dict':
-                    current_arg.update({'type': utils.str2dict})
-                continue
+                if current_type_str in allowed_type_dict:
+                    current_arg['type'] = allowed_type_dict[current_type_str]
+                    continue
+                else:
+                    raise exceptions.CommandError(
+                        _("Invalid value_specs {valspec}: type {curtypestr}"
+                            " is not supported").format(
+                            valspec=' '.join(values_specs),
+                            curtypestr=current_type_str))
+
         elif _item == 'list=true':
             _list_flag = True
             continue
@@ -319,7 +252,6 @@ def parse_args_to_dict(values_specs):
         if _item.startswith('---'):
             raise exceptions.CommandError(
                 _("Invalid values_specs %s") % ' '.join(values_specs))
-
         _values_specs.append(_item)
 
     # Deal with last one argument
@@ -415,7 +347,7 @@ class NeutronCommand(command.Command):
         parser = super(NeutronCommand, self).get_parser(prog_name)
         parser.add_argument(
             '--request-format',
-            help=_('DEPRECATED! Only JSON request format is supported.'),
+            help=argparse.SUPPRESS,
             default='json',
             choices=['json', ], )
         parser.add_argument(
@@ -429,7 +361,6 @@ class NeutronCommand(command.Command):
         pass
 
     def format_output_data(self, data):
-        self.cleanup_output_data(data)
         # Modify data to make it more readable
         if self.resource in data:
             for k, v in six.iteritems(data[self.resource]):
@@ -484,11 +415,14 @@ class CreateCommand(NeutronCommand, show.ShowOne):
             data = obj_creator(self.parent_id, body)
         else:
             data = obj_creator(body)
-        self.format_output_data(data)
+        self.cleanup_output_data(data)
+        if parsed_args.formatter == 'table':
+            self.format_output_data(data)
         info = self.resource in data and data[self.resource] or None
         if info:
-            print(_('Created a new %s:') % self.resource,
-                  file=self.app.stdout)
+            if parsed_args.formatter == 'table':
+                print(_('Created a new %s:') % self.resource,
+                      file=self.app.stdout)
         else:
             info = {'': ''}
         return zip(*sorted(six.iteritems(info)))
@@ -556,17 +490,19 @@ class DeleteCommand(NeutronCommand):
     log = None
     allow_names = True
     help_resource = None
+    bulk_delete = True
 
     def get_parser(self, prog_name):
         parser = super(DeleteCommand, self).get_parser(prog_name)
-        if self.allow_names:
-            help_str = _('ID or name of %s to delete.')
-        else:
-            help_str = _('ID of %s to delete.')
         if not self.help_resource:
             self.help_resource = self.resource
+        if self.allow_names:
+            help_str = _('ID(s) or name(s) of %s to delete.')
+        else:
+            help_str = _('ID(s) of %s to delete.')
         parser.add_argument(
             'id', metavar=self.resource.upper(),
+            nargs='+' if self.bulk_delete else 1,
             help=help_str % self.help_resource)
         self.add_known_arguments(parser)
         return parser
@@ -576,24 +512,64 @@ class DeleteCommand(NeutronCommand):
         neutron_client = self.get_client()
         obj_deleter = getattr(neutron_client,
                               "delete_%s" % self.cmd_resource)
+
+        if self.bulk_delete:
+            self._bulk_delete(obj_deleter, neutron_client, parsed_args.id)
+        else:
+            self.delete_item(obj_deleter, neutron_client, parsed_args.id)
+            print((_('Deleted %(resource)s: %(id)s')
+                   % {'id': parsed_args.id,
+                      'resource': self.resource}),
+                  file=self.app.stdout)
+        return
+
+    def _bulk_delete(self, obj_deleter, neutron_client, parsed_args_ids):
+        successful_delete = []
+        non_existent = []
+        multiple_ids = []
+        for item_id in parsed_args_ids:
+            try:
+                self.delete_item(obj_deleter, neutron_client, item_id)
+                successful_delete.append(item_id)
+            except exceptions.NotFound:
+                non_existent.append(item_id)
+            except exceptions.NeutronClientNoUniqueMatch:
+                multiple_ids.append(item_id)
+        if successful_delete:
+            print((_('Deleted %(resource)s(s): %(id)s'))
+                  % {'id': ", ".join(successful_delete),
+                     'resource': self.cmd_resource},
+                  file=self.app.stdout)
+        if non_existent or multiple_ids:
+            err_msgs = []
+            if non_existent:
+                err_msgs.append((_("Unable to find %(resource)s(s) with id(s) "
+                                   "'%(id)s'.") %
+                                 {'resource': self.cmd_resource,
+                                  'id': ", ".join(non_existent)}))
+            if multiple_ids:
+                err_msgs.append((_("Multiple %(resource)s(s) matches found "
+                                   "for name(s) '%(id)s'. Please use an ID "
+                                   "to be more specific.") %
+                                 {'resource': self.cmd_resource,
+                                  'id': ", ".join(multiple_ids)}))
+            raise exceptions.NeutronCLIError(message='\n'.join(err_msgs))
+
+    def delete_item(self, obj_deleter, neutron_client, item_id):
         if self.allow_names:
             params = {'cmd_resource': self.cmd_resource,
                       'parent_id': self.parent_id}
             _id = find_resourceid_by_name_or_id(neutron_client,
                                                 self.resource,
-                                                parsed_args.id,
+                                                item_id,
                                                 **params)
         else:
-            _id = parsed_args.id
+            _id = item_id
 
         if self.parent_id:
             obj_deleter(_id, self.parent_id)
         else:
             obj_deleter(_id)
-        print((_('Deleted %(resource)s: %(id)s')
-               % {'id': parsed_args.id,
-                  'resource': self.resource}),
-              file=self.app.stdout)
         return
 
 
@@ -694,8 +670,7 @@ class ListCommand(NeutronCommand, lister.Lister):
         return search_opts
 
     def call_server(self, neutron_client, search_opts, parsed_args):
-        resource_plural = _get_resource_plural(self.cmd_resource,
-                                               neutron_client)
+        resource_plural = neutron_client.get_resource_plural(self.cmd_resource)
         obj_lister = getattr(neutron_client, "list_%s" % resource_plural)
         if self.parent_id:
             data = obj_lister(self.parent_id, **search_opts)
@@ -728,7 +703,7 @@ class ListCommand(NeutronCommand, lister.Lister):
             if dirs:
                 search_opts.update({'sort_dir': dirs})
         data = self.call_server(neutron_client, search_opts, parsed_args)
-        collection = _get_resource_plural(self.resource, neutron_client)
+        collection = neutron_client.get_resource_plural(self.resource)
         return data.get(collection, [])
 
     def extend_list(self, data, parsed_args):
@@ -751,16 +726,48 @@ class ListCommand(NeutronCommand, lister.Lister):
             # if no -c(s) by user and list_columns, we use columns in
             # both list_columns and returned resource.
             # Also Keep their order the same as in list_columns
-            _columns = [x for x in self.list_columns if x in _columns]
+            _columns = self._setup_columns_with_tenant_id(self.list_columns,
+                                                          _columns)
 
-        formatters = self._formatters
-        if hasattr(self, '_formatters_csv') and parsed_args.formatter == 'csv':
+        if parsed_args.formatter == 'table':
+            formatters = self._formatters
+        elif (parsed_args.formatter == 'csv' and
+              hasattr(self, '_formatters_csv')):
             formatters = self._formatters_csv
+        else:
+            # For other formatters, we use raw value returned from neutron
+            formatters = {}
 
         return (_columns, (utils.get_item_properties(
             s, _columns, formatters=formatters, )
             for s in info), )
 
+    def _setup_columns_with_tenant_id(self, display_columns, avail_columns):
+        _columns = [x for x in display_columns if x in avail_columns]
+        if 'tenant_id' in display_columns:
+            return _columns
+        if 'tenant_id' not in avail_columns:
+            return _columns
+        if not self.is_admin_role():
+            return _columns
+        try:
+            pos_id = _columns.index('id')
+        except ValueError:
+            pos_id = 0
+        try:
+            pos_name = _columns.index('name')
+        except ValueError:
+            pos_name = 0
+        _columns.insert(max(pos_id, pos_name) + 1, 'tenant_id')
+        return _columns
+
+    def is_admin_role(self):
+        client = self.get_client()
+        auth_ref = client.httpclient.get_auth_ref()
+        if not auth_ref:
+            return False
+        return 'admin' in auth_ref.role_names
+
     def take_action(self, parsed_args):
         self.set_extra_attrs(parsed_args)
         data = self.retrieve_list(parsed_args)
@@ -813,7 +820,9 @@ class ShowCommand(NeutronCommand, show.ShowOne):
             data = obj_shower(_id, self.parent_id, **params)
         else:
             data = obj_shower(_id, **params)
-        self.format_output_data(data)
+        self.cleanup_output_data(data)
+        if parsed_args.formatter == 'table':
+            self.format_output_data(data)
         resource = data[self.resource]
         if self.resource in data:
             return zip(*sorted(six.iteritems(resource)))
