@@ -16,7 +16,7 @@
 import itertools
 import sys
 
-from mox3 import mox
+import mock
 from oslo_serialization import jsonutils
 
 from neutronclient.common import exceptions
@@ -25,74 +25,57 @@ from neutronclient import shell
 from neutronclient.tests.unit import test_cli20
 
 
-class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
+class CLITestV20CreateNetworkJSON(test_cli20.CLITestV20Base):
     def setUp(self):
-        super(CLITestV20NetworkJSON, self).setUp(plurals={'tags': 'tag'})
+        super(CLITestV20CreateNetworkJSON, self).setUp(plurals={'tags': 'tag'})
+
+    def _test_create_network(self, **kwargs):
+        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
+        resource = kwargs.pop('resource', 'network')
+
+        name = kwargs.pop('name', 'myname')
+        myid = kwargs.pop('myid', 'myid')
+        args = kwargs.pop('args', [name, ])
+        position_names = kwargs.pop('position_names', ['name', ])
+        position_values = kwargs.pop('position_values', [name, ])
+
+        self._test_create_resource(resource, cmd, name, myid, args,
+                                   position_names, position_values,
+                                   **kwargs)
 
     def test_create_network(self):
         # Create net: myname.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
-        name = 'myname'
-        myid = 'myid'
-        args = [name, ]
-        position_names = ['name', ]
-        position_values = [name, ]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values)
+        self._test_create_network()
 
     def test_create_network_with_unicode(self):
         # Create net: u'\u7f51\u7edc'.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
-        name = u'\u7f51\u7edc'
-        myid = 'myid'
-        args = [name, ]
-        position_names = ['name', ]
-        position_values = [name, ]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values)
+        self._test_create_network(name=u'\u7f51\u7edc')
 
     def test_create_network_description(self):
         # Create net: --tenant_id tenantid myname.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
         name = 'myname'
-        myid = 'myid'
         args = ['--description', 'Nice network', name]
-        position_names = ['name', ]
-        position_values = [name, ]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values,
-                                   description='Nice network')
+        self._test_create_network(name=name,
+                                  args=args,
+                                  description='Nice network')
 
-    def test_create_network_tenant(self):
+    def test_create_network_tenant_underscore(self):
         # Create net: --tenant_id tenantid myname.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
         name = 'myname'
-        myid = 'myid'
         args = ['--tenant_id', 'tenantid', name]
-        position_names = ['name', ]
-        position_values = [name, ]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values,
-                                   tenant_id='tenantid')
+        self._test_create_network(name=name, args=args, tenant_id="tenantid")
 
+    def test_create_network_tenant_dash(self):
         # Test dashed options
+        # Create net: --tenant_id tenantid myname.
+        name = 'myname'
         args = ['--tenant-id', 'tenantid', name]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values,
-                                   tenant_id='tenantid')
+        self._test_create_network(name=name, args=args, tenant_id="tenantid")
 
     def test_create_network_provider_args(self):
         # Create net: with --provider arguments.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
-        name = 'myname'
-        myid = 'myid'
-
         # Test --provider attributes before network name
+        name = 'myname'
         args = ['--provider:network_type', 'vlan',
                 '--provider:physical_network', 'physnet1',
                 '--provider:segmentation_id', '400', name]
@@ -100,123 +83,108 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
                           'provider:physical_network',
                           'provider:segmentation_id', 'name']
         position_values = ['vlan', 'physnet1', '400', name]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values)
+        self._test_create_network(name=name,
+                                  args=args,
+                                  position_names=position_names,
+                                  position_values=position_values)
 
     def test_create_network_tags(self):
         # Create net: myname --tags a b.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
         name = 'myname'
-        myid = 'myid'
         args = [name, '--tags', 'a', 'b']
-        position_names = ['name', ]
-        position_values = [name, ]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values,
-                                   tags=['a', 'b'])
+        self._test_create_network(name=name, args=args, tags=['a', 'b'])
 
-    def test_create_network_state(self):
+    def test_create_network_state_underscore(self):
         # Create net: --admin_state_down myname.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
         name = 'myname'
-        myid = 'myid'
         args = ['--admin_state_down', name, ]
-        position_names = ['name', ]
-        position_values = [name, ]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values,
-                                   admin_state_up=False)
+        self._test_create_network(name=name, args=args, admin_state_up=False)
 
+    def test_create_network_state_dash(self):
         # Test dashed options
+        name = 'myname'
         args = ['--admin-state-down', name, ]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values,
-                                   admin_state_up=False)
+        self._test_create_network(name=name, args=args, admin_state_up=False)
 
     def test_create_network_vlan_transparent(self):
         # Create net: myname --vlan-transparent True.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
         name = 'myname'
-        myid = 'myid'
         args = ['--vlan-transparent', 'True', name]
-        vlantrans = {'vlan_transparent': 'True'}
-        position_names = ['name', ]
-        position_values = [name, ]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values,
-                                   **vlantrans)
+        self._test_create_network(name=name,
+                                  args=args,
+                                  vlan_transparent='True')
 
     def test_create_network_with_qos_policy(self):
         # Create net: --qos-policy mypolicy.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
         name = 'myname'
-        myid = 'myid'
         qos_policy_name = 'mypolicy'
         args = [name, '--qos-policy', qos_policy_name]
         position_names = ['name', 'qos_policy_id']
         position_values = [name, qos_policy_name]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values)
+        self._test_create_network(name=name,
+                                  args=args,
+                                  position_names=position_names,
+                                  position_values=position_values)
 
     def test_create_network_with_az_hint(self):
         # Create net: --availability-zone-hint zone1
         # --availability-zone-hint zone2.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
         name = 'myname'
-        myid = 'myid'
         args = ['--availability-zone-hint', 'zone1',
                 '--availability-zone-hint', 'zone2', name]
         position_names = ['availability_zone_hints', 'name']
         position_values = [['zone1', 'zone2'], name]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values)
+        self._test_create_network(name=name,
+                                  args=args,
+                                  position_names=position_names,
+                                  position_values=position_values)
 
     def test_create_network_with_dns_domain(self):
         # Create net: --dns-domain my-domain.org.
-        resource = 'network'
-        cmd = network.CreateNetwork(test_cli20.MyApp(sys.stdout), None)
         name = 'myname'
-        myid = 'myid'
         dns_domain_name = 'my-domain.org.'
         args = [name, '--dns-domain', dns_domain_name]
         position_names = ['name', 'dns_domain']
         position_values = [name, dns_domain_name]
-        self._test_create_resource(resource, cmd, name, myid, args,
-                                   position_names, position_values)
+        self._test_create_network(name=name,
+                                  args=args,
+                                  position_names=position_names,
+                                  position_values=position_values)
+
+
+class CLITestV20ListNetworkJSON(test_cli20.CLITestV20Base):
+    def setUp(self):
+        super(CLITestV20ListNetworkJSON, self).setUp(plurals={'tags': 'tag'})
 
     def test_list_nets_empty_with_column(self):
         resources = "networks"
         cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
-        self.mox.StubOutWithMock(cmd, "get_client")
-        self.mox.StubOutWithMock(self.client.httpclient, "request")
-        self.mox.StubOutWithMock(network.ListNetwork, "extend_list")
-        network.ListNetwork.extend_list(mox.IsA(list), mox.IgnoreArg())
-        cmd.get_client().MultipleTimes().AndReturn(self.client)
         reses = {resources: []}
         resstr = self.client.serialize(reses)
+        resp = (test_cli20.MyResp(200), resstr)
         # url method body
         query = "id=myfakeid"
         args = ['-c', 'id', '--', '--id', 'myfakeid']
         path = getattr(self.client, resources + "_path")
-        self.client.httpclient.request(
+        with mock.patch.object(cmd, "get_client",
+                               return_value=self.client) as mock_get_client, \
+                mock.patch.object(self.client.httpclient, "request",
+                                  return_value=resp) as mock_request, \
+                mock.patch.object(network.ListNetwork, "extend_list",
+                                  return_value=None) as mock_extend_list:
+            cmd_parser = cmd.get_parser("list_" + resources)
+            shell.run_command(cmd, cmd_parser, args)
+
+        mock_get_client.assert_called_once_with()
+        mock_request.assert_called_once_with(
             test_cli20.MyUrlComparator(test_cli20.end_url(path, query),
                                        self.client),
             'GET',
             body=None,
-            headers=mox.ContainsKeyValue(
-                'X-Auth-Token',
-                test_cli20.TOKEN)).AndReturn(
-                    (test_cli20.MyResp(200), resstr))
-        self.mox.ReplayAll()
-        cmd_parser = cmd.get_parser("list_" + resources)
-        shell.run_command(cmd, cmd_parser, args)
-        self.mox.VerifyAll()
-        self.mox.UnsetStubs()
+            headers=test_cli20.ContainsKeyValue(
+                {'X-Auth-Token': test_cli20.TOKEN}))
+        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
+                                                 mock.ANY)
         _str = self.fake_stdout.make_string()
         self.assertEqual('\n', _str)
 
@@ -225,18 +193,22 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
                             sort_key=(), sort_dir=(), base_args=None,
                             query=''):
         resources = "networks"
-        self.mox.StubOutWithMock(network.ListNetwork, "extend_list")
-        network.ListNetwork.extend_list(mox.IsA(list), mox.IgnoreArg())
-        self._test_list_resources(resources, cmd, detail, tags,
-                                  fields_1, fields_2, page_size=page_size,
-                                  sort_key=sort_key, sort_dir=sort_dir,
-                                  base_args=base_args, query=query)
+        with mock.patch.object(network.ListNetwork, "extend_list",
+                               return_value=None) as mock_extend_list:
+            self._test_list_resources(resources, cmd, detail, tags,
+                                      fields_1, fields_2, page_size=page_size,
+                                      sort_key=sort_key, sort_dir=sort_dir,
+                                      base_args=base_args, query=query)
+        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
+                                                 mock.ANY)
 
     def test_list_nets_pagination(self):
         cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
-        self.mox.StubOutWithMock(network.ListNetwork, "extend_list")
-        network.ListNetwork.extend_list(mox.IsA(list), mox.IgnoreArg())
-        self._test_list_resources_with_pagination("networks", cmd)
+        with mock.patch.object(network.ListNetwork, "extend_list",
+                               return_value=None) as mock_extend_list:
+            self._test_list_resources_with_pagination("networks", cmd)
+        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
+                                                 mock.ANY)
 
     def test_list_nets_sort(self):
         # list nets:
@@ -283,42 +255,51 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
         self._test_list_networks(cmd, detail=True, tags=['a', 'b'])
 
     def _test_list_nets_extend_subnets(self, data, expected):
-        def setup_list_stub(resources, data, query):
-            reses = {resources: data}
-            resstr = self.client.serialize(reses)
-            resp = (test_cli20.MyResp(200), resstr)
-            path = getattr(self.client, resources + '_path')
-            self.client.httpclient.request(
-                test_cli20.MyUrlComparator(
-                    test_cli20.end_url(path, query), self.client),
-                'GET',
-                body=None,
-                headers=mox.ContainsKeyValue(
-                    'X-Auth-Token', test_cli20.TOKEN)).AndReturn(resp)
-
         cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
-        self.mox.StubOutWithMock(cmd, 'get_client')
-        self.mox.StubOutWithMock(self.client.httpclient, 'request')
-        cmd.get_client().AndReturn(self.client)
-        setup_list_stub('networks', data, '')
-        cmd.get_client().AndReturn(self.client)
+        nets_path = getattr(self.client, 'networks_path')
+        subnets_path = getattr(self.client, 'subnets_path')
+        nets_query = ''
         filters = ''
         for n in data:
             for s in n['subnets']:
                 filters = filters + "&id=%s" % s
-        setup_list_stub('subnets',
-                        [{'id': 'mysubid1', 'cidr': '192.168.1.0/24'},
+        subnets_query = 'fields=id&fields=cidr' + filters
+        with mock.patch.object(cmd, 'get_client',
+                               return_value=self.client) as mock_get_client, \
+                mock.patch.object(self.client.httpclient,
+                                  "request") as mock_request:
+            resp1 = (test_cli20.MyResp(200),
+                     self.client.serialize({'networks': data}))
+            resp2 = (test_cli20.MyResp(200),
+                     self.client.serialize({'subnets': [
+                         {'id': 'mysubid1', 'cidr': '192.168.1.0/24'},
                          {'id': 'mysubid2', 'cidr': '172.16.0.0/24'},
-                         {'id': 'mysubid3', 'cidr': '10.1.1.0/24'}],
-                        query='fields=id&fields=cidr' + filters)
-        self.mox.ReplayAll()
+                         {'id': 'mysubid3', 'cidr': '10.1.1.0/24'}]}))
+            mock_request.side_effect = [resp1, resp2]
+            args = []
+            cmd_parser = cmd.get_parser('list_networks')
+            parsed_args = cmd_parser.parse_args(args)
+            result = cmd.take_action(parsed_args)
 
-        args = []
-        cmd_parser = cmd.get_parser('list_networks')
-        parsed_args = cmd_parser.parse_args(args)
-        result = cmd.take_action(parsed_args)
-        self.mox.VerifyAll()
-        self.mox.UnsetStubs()
+        mock_get_client.assert_called_with()
+        self.assertEqual(2, mock_request.call_count)
+        mock_request.assert_has_calls([
+            mock.call(
+                test_cli20.MyUrlComparator(
+                    test_cli20.end_url(nets_path, nets_query),
+                    self.client),
+                'GET',
+                body=None,
+                headers=test_cli20.ContainsKeyValue(
+                    {'X-Auth-Token': test_cli20.TOKEN})),
+            mock.call(
+                test_cli20.MyUrlComparator(
+                    test_cli20.end_url(subnets_path, subnets_query),
+                    self.client),
+                'GET',
+                body=None,
+                headers=test_cli20.ContainsKeyValue(
+                    {'X-Auth-Token': test_cli20.TOKEN}))])
         _result = [x for x in result[1]]
         self.assertEqual(len(expected), len(_result))
         for res, exp in zip(_result, expected):
@@ -353,9 +334,11 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
     def _test_list_nets_columns(self, cmd, returned_body,
                                 args=('-f', 'json')):
         resources = 'networks'
-        self.mox.StubOutWithMock(network.ListNetwork, "extend_list")
-        network.ListNetwork.extend_list(mox.IsA(list), mox.IgnoreArg())
-        self._test_list_columns(cmd, resources, returned_body, args=args)
+        with mock.patch.object(network.ListNetwork, "extend_list",
+                               return_value=None) as mock_extend_list:
+            self._test_list_columns(cmd, resources, returned_body, args=args)
+        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
+                                                 mock.ANY)
 
     def test_list_nets_defined_column(self):
         cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
@@ -389,46 +372,43 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
     def test_list_external_nets_empty_with_column(self):
         resources = "networks"
         cmd = network.ListExternalNetwork(test_cli20.MyApp(sys.stdout), None)
-        self.mox.StubOutWithMock(cmd, "get_client")
-        self.mox.StubOutWithMock(self.client.httpclient, "request")
-        self.mox.StubOutWithMock(network.ListNetwork, "extend_list")
-        network.ListNetwork.extend_list(mox.IsA(list), mox.IgnoreArg())
-        cmd.get_client().MultipleTimes().AndReturn(self.client)
         reses = {resources: []}
         resstr = self.client.serialize(reses)
         # url method body
         query = "router%3Aexternal=True&id=myfakeid"
         args = ['-c', 'id', '--', '--id', 'myfakeid']
         path = getattr(self.client, resources + "_path")
-        self.client.httpclient.request(
+        resp = (test_cli20.MyResp(200), resstr)
+        with mock.patch.object(cmd, "get_client",
+                               return_value=self.client) as mock_get_client, \
+                mock.patch.object(self.client.httpclient, "request",
+                                  return_value=resp) as mock_request, \
+                mock.patch.object(network.ListNetwork, "extend_list",
+                                  return_value=None) as mock_extend_list:
+            cmd_parser = cmd.get_parser("list_" + resources)
+            shell.run_command(cmd, cmd_parser, args)
+
+        mock_get_client.assert_called_once_with()
+        mock_request.assert_called_once_with(
             test_cli20.MyUrlComparator(
                 test_cli20.end_url(path, query), self.client),
             'GET',
             body=None,
-            headers=mox.ContainsKeyValue(
-                'X-Auth-Token',
-                test_cli20.TOKEN)).AndReturn(
-                    (test_cli20.MyResp(200), resstr))
-        self.mox.ReplayAll()
-        cmd_parser = cmd.get_parser("list_" + resources)
-        shell.run_command(cmd, cmd_parser, args)
-        self.mox.VerifyAll()
-        self.mox.UnsetStubs()
+            headers=test_cli20.ContainsKeyValue(
+                {'X-Auth-Token': test_cli20.TOKEN}))
+        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
+                                                 mock.ANY)
         _str = self.fake_stdout.make_string()
         self.assertEqual('\n', _str)
 
     def _test_list_external_nets(self, resources, cmd,
                                  detail=False, tags=(),
                                  fields_1=(), fields_2=()):
-        self.mox.StubOutWithMock(cmd, "get_client")
-        self.mox.StubOutWithMock(self.client.httpclient, "request")
-        self.mox.StubOutWithMock(network.ListNetwork, "extend_list")
-        network.ListNetwork.extend_list(mox.IsA(list), mox.IgnoreArg())
-        cmd.get_client().MultipleTimes().AndReturn(self.client)
         reses = {resources: [{'id': 'myid1', },
                              {'id': 'myid2', }, ], }
 
         resstr = self.client.serialize(reses)
+        resp = (test_cli20.MyResp(200), resstr)
 
         # url method body
         query = ""
@@ -466,18 +446,25 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
             query = query and query + '&verbose=True' or 'verbose=True'
         path = getattr(self.client, resources + "_path")
 
-        self.client.httpclient.request(
+        with mock.patch.object(cmd, "get_client",
+                               return_value=self.client) as mock_get_client, \
+                mock.patch.object(self.client.httpclient, "request",
+                                  return_value=resp) as mock_request, \
+                mock.patch.object(network.ListNetwork, "extend_list",
+                                  return_value=None) as mock_extend_list:
+            cmd_parser = cmd.get_parser("list_" + resources)
+            shell.run_command(cmd, cmd_parser, args)
+
+        mock_get_client.assert_called_once_with()
+        mock_request.assert_called_once_with(
             test_cli20.MyUrlComparator(
                 test_cli20.end_url(path, query), self.client),
             'GET',
             body=None,
-            headers=mox.ContainsKeyValue('X-Auth-Token', test_cli20.TOKEN)
-        ).AndReturn((test_cli20.MyResp(200), resstr))
-        self.mox.ReplayAll()
-        cmd_parser = cmd.get_parser("list_" + resources)
-        shell.run_command(cmd, cmd_parser, args)
-        self.mox.VerifyAll()
-        self.mox.UnsetStubs()
+            headers=test_cli20.ContainsKeyValue(
+                {'X-Auth-Token': test_cli20.TOKEN}))
+        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
+                                                 mock.ANY)
         _str = self.fake_stdout.make_string()
 
         self.assertIn('myid1', _str)
@@ -502,7 +489,7 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
         self._test_list_external_nets(resources, cmd,
                                       detail=True, tags=['a', 'b'])
 
-    def test_list_externel_nets_fields(self):
+    def test_list_external_nets_fields(self):
         # List external nets: --fields a --fields b -- --fields c d.
         resources = "networks"
         cmd = network.ListExternalNetwork(test_cli20.MyApp(sys.stdout), None)
@@ -510,6 +497,14 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
                                       fields_1=['a', 'b'],
                                       fields_2=['c', 'd'])
 
+    def test_list_shared_networks(self):
+        # list nets : --shared False
+        cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
+        self._test_list_networks(cmd, base_args='--shared False'.split(),
+                                 query='shared=False')
+
+
+class CLITestV20UpdateNetworkJSON(test_cli20.CLITestV20Base):
     def test_update_network_exception(self):
         # Update net: myid.
         resource = 'network'
@@ -572,6 +567,8 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
                                    ['myid', '--no-dns-domain'],
                                    {'dns_domain': "", })
 
+
+class CLITestV20ShowNetworkJSON(test_cli20.CLITestV20Base):
     def test_show_network(self):
         # Show net: --fields id --fields name myid.
         resource = 'network'
@@ -580,6 +577,8 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
         self._test_show_resource(resource, cmd, self.test_id, args,
                                  ['id', 'name'])
 
+
+class CLITestV20DeleteNetworkJSON(test_cli20.CLITestV20Base):
     def test_delete_network(self):
         # Delete net: myid.
         resource = 'network'
@@ -597,21 +596,20 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
         args = [myid1, myid2]
         self._test_delete_resource(resource, cmd, myid1, args, extra_id=myid2)
 
-    def _test_extend_list(self, mox_calls):
-        data = [{'id': 'netid%d' % i, 'name': 'net%d' % i,
-                 'subnets': ['mysubid%d' % i]}
-                for i in range(10)]
-        self.mox.StubOutWithMock(self.client.httpclient, "request")
-        path = getattr(self.client, 'subnets_path')
-        cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
-        self.mox.StubOutWithMock(cmd, "get_client")
-        cmd.get_client().MultipleTimes().AndReturn(self.client)
-        mox_calls(path, data)
-        self.mox.ReplayAll()
-        known_args, _vs = cmd.get_parser('create_subnets').parse_known_args()
-        cmd.extend_list(data, known_args)
-        self.mox.VerifyAll()
+    def test_bulk_delete_network_fail(self):
+        # Delete net: myid1 myid2.
+        resource = 'network'
+        cmd = network.DeleteNetwork(test_cli20.MyApp(sys.stdout), None)
+        myid1 = 'myid1'
+        myid2 = 'myid2'
+        args = [myid1, myid2]
+        self.assertRaises(exceptions.NeutronCLIError,
+                          self._test_delete_resource,
+                          resource, cmd, myid1, args, extra_id=myid2,
+                          delete_fail=True)
 
+
+class CLITestV20ExtendListNetworkJSON(test_cli20.CLITestV20Base):
     def _build_test_data(self, data):
         subnet_ids = []
         response = []
@@ -628,47 +626,69 @@ class CLITestV20NetworkJSON(test_cli20.CLITestV20Base):
         return filters, resp
 
     def test_extend_list(self):
-        def mox_calls(path, data):
-            filters, response = self._build_test_data(data)
-            self.client.httpclient.request(
-                test_cli20.MyUrlComparator(test_cli20.end_url(
-                    path, 'fields=id&fields=cidr' + filters), self.client),
-                'GET',
-                body=None,
-                headers=mox.ContainsKeyValue(
-                    'X-Auth-Token', test_cli20.TOKEN)).AndReturn(response)
+        data = [{'id': 'netid%d' % i, 'name': 'net%d' % i,
+                 'subnets': ['mysubid%d' % i]}
+                for i in range(10)]
+        filters, response = self._build_test_data(data)
+        path = getattr(self.client, 'subnets_path')
+        cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
+        with mock.patch.object(cmd, "get_client",
+                               return_value=self.client) as mock_get_client, \
+                mock.patch.object(self.client.httpclient, "request",
+                                  return_value=response) as mock_request:
+            known_args, _vs = cmd.get_parser('create_subnets')\
+                .parse_known_args()
+            cmd.extend_list(data, known_args)
 
-        self._test_extend_list(mox_calls)
+        mock_get_client.assert_called_once_with()
+        mock_request.assert_called_once_with(
+            test_cli20.MyUrlComparator(test_cli20.end_url(
+                path, 'fields=id&fields=cidr' + filters), self.client),
+            'GET',
+            body=None,
+            headers=test_cli20.ContainsKeyValue(
+                {'X-Auth-Token': test_cli20.TOKEN}))
 
     def test_extend_list_exceed_max_uri_len(self):
-        def mox_calls(path, data):
-            sub_data_lists = [data[:len(data) - 1], data[len(data) - 1:]]
-            filters, response = self._build_test_data(data)
-
-            # 1 char of extra URI len will cause a split in 2 requests
-            self.mox.StubOutWithMock(self.client.httpclient,
-                                     "_check_uri_length")
-            self.client.httpclient._check_uri_length(mox.IgnoreArg()).AndRaise(
-                exceptions.RequestURITooLong(excess=1))
-
-            for data in sub_data_lists:
-                filters, response = self._build_test_data(data)
-                self.client.httpclient._check_uri_length(
-                    mox.IgnoreArg()).AndReturn(None)
-                self.client.httpclient.request(
-                    test_cli20.MyUrlComparator(
-                        test_cli20.end_url(
-                            path, 'fields=id&fields=cidr%s' % filters),
-                        self.client),
-                    'GET',
-                    body=None,
-                    headers=mox.ContainsKeyValue(
-                        'X-Auth-Token', test_cli20.TOKEN)).AndReturn(response)
-
-        self._test_extend_list(mox_calls)
-
-    def test_list_shared_networks(self):
-        # list nets : --shared False
+        data = [{'id': 'netid%d' % i, 'name': 'net%d' % i,
+                 'subnets': ['mysubid%d' % i]}
+                for i in range(10)]
+        filters1, response1 = self._build_test_data(data[:len(data) - 1])
+        filters2, response2 = self._build_test_data(data[len(data) - 1:])
+        path = getattr(self.client, 'subnets_path')
         cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
-        self._test_list_networks(cmd, base_args='--shared False'.split(),
-                                 query='shared=False')
+        with mock.patch.object(cmd, "get_client",
+                               return_value=self.client) as mock_get_client, \
+                mock.patch.object(self.client.httpclient,
+                                  "request") as mock_request, \
+                mock.patch.object(self.client.httpclient, "_check_uri_length",
+                                  return_value=None) as mock_check_uri_length:
+            # 1 char of extra URI len will cause a split in 2 requests
+            mock_check_uri_length.side_effect = [
+                exceptions.RequestURITooLong(excess=1), None, None]
+            mock_request.side_effect = [response1, response2]
+            known_args, _vs = cmd.get_parser('create_subnets')\
+                .parse_known_args()
+            cmd.extend_list(data, known_args)
+
+        mock_get_client.assert_called_once_with()
+        self.assertEqual(2, mock_request.call_count)
+        mock_request.assert_has_calls([
+            mock.call(
+                test_cli20.MyUrlComparator(
+                    test_cli20.end_url(
+                        path, 'fields=id&fields=cidr%s' % filters1),
+                    self.client),
+                'GET',
+                body=None,
+                headers=test_cli20.ContainsKeyValue(
+                    {'X-Auth-Token': test_cli20.TOKEN})),
+            mock.call(
+                test_cli20.MyUrlComparator(
+                    test_cli20.end_url(
+                        path, 'fields=id&fields=cidr%s' % filters2),
+                    self.client),
+                'GET',
+                body=None,
+                headers=test_cli20.ContainsKeyValue(
+                    {'X-Auth-Token': test_cli20.TOKEN}))])
